#========================================================================================
# package      DE_EPAGES::Catalog::t::unit::CategoryService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Catalog::API::CategoryService
#========================================================================================
package DE_EPAGES::Catalog::t::unit::CategoryService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_createCategory();
    $self->test_updateCategory();
    $self->test_getCategoryData();

    done_testing();
}



sub test_createCategory {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CategoryService = $Mockery->get('SUT');

    my $Category = $Mockery->get('Category');
    my $hUpdateRequestData = {
        'alias' => 'alias',
        'name' => 'name',
        'pageTitle' => 'title',
        'navigationCaption' => 'caption',
        'description' => 'description',
    };

    $CategoryService->createCategory($Category, 'de_DE', $hUpdateRequestData);

    ok(WasCalled($Category, 'renumberChildren'), "$SubTestName - test if order of categories was corrected after new subcategory was created.");
    my $NewCategory = $Mockery->get('NewCategory');
    is($NewCategory->get('Name'), 'name', "$SubTestName - test if name was correctly set.");
    is($NewCategory->get('NavigationCaption'), 'caption', "$SubTestName - test if navigation caption was correctly set.");
    my $CategoryPackage = $Mockery->get('CategoryPackage');
    my $aInsertParams = GetParametersFromMockifyCall($CategoryPackage, 'insert');
    my $hCategoryInsertData = $aInsertParams->[0];
    is($hCategoryInsertData->{'Alias'}, 'alias', "$SubTestName - test if parameter correctly set.");
    is($hCategoryInsertData->{'IsVisible'}, '0', "$SubTestName - test if parameter correctly set.");
    is($hCategoryInsertData->{'VisibleInNavigation'}, '1', "$SubTestName - test if parameter correctly set.");
    is($hCategoryInsertData->{'Site'}->alias(), 'TestShop', "$SubTestName - test if parameter correctly set.");
    is($hCategoryInsertData->{'Parent'}->alias(), 'parentAlias', "$SubTestName - test if parameter correctly set.");
    is($hCategoryInsertData->{'Class'}->alias(), 'CategoryClass', "$SubTestName - test if parameter correctly set.");
}



sub test_updateCategory {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $CategoryService = $Mockery->get('SUT');

    my $Category = $Mockery->get('Category');
    my $hUpdateRequestData = {
        'alias' => 'alias2',
        'name' => 'name2',
        'pageTitle' => 'title2',
        'description' => 'description2',
        'navigationCaption' => 'caption2',
    };

    $CategoryService->updateCategory($Category, 'de_DE', $hUpdateRequestData);

    is($Category->get('Name'), 'name2', "$SubTestName - test if category name was updated");
    is($Category->get('Alias'), 'alias2', "$SubTestName - test if category alias  was updated");
    is($Category->get('Title'), 'title2', "$SubTestName - test if category title  was updated");
    is($Category->get('NavigationCaption'), 'caption2', "$SubTestName - test if category navigation caption  was updated");
    is($Category->get('Description'), 'description2', "$SubTestName - test if category description was updated");
}



sub test_getCategoryData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks_getCategoryData();
    my $CategoryService = $Mockery->get('SUT');

    my $Category = $Mockery->get('Category');
    my $Context = $Mockery->get('Context');

    my $hCategoryData = $CategoryService->getCategoryData($Category, 'de_DE', $Context);

    my $hExpectedCategoryData = $self->_createExpectedCategoryData();
    is_deeply($hCategoryData, $hExpectedCategoryData, "$SubTestName - test if expected category data was returned.");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
    $Mockery->extend($self->_createCommonMocks());

    $Mockery->register('SUT', ['Locale', 'Class'], sub {
        my ($Locale, $Class) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Catalog::API::CategoryService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocale', $Locale, ['string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_LoadClassByAlias', $Class, [{'string'=>'Category'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Category', ['Shop', 'Class'], sub {
        my ($Shop, $Class) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'parentAlias',
            'GUID' => '1212121212',
            'Site' => $Shop,
            'instanceOf' => 'Category',
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('uniqueChildAlias', 'alias', [{'string' => 'alias'}]);
        $Mockify->addMockWithReturnValue('class', $Class);
        $Mockify->addMockWithReturnValue('renumberChildren', undef);
        return $Mockify->getMockObject();
    });
    $Mockery->register('CategoryPackage', ['NewCategory'], sub {
        my ($NewCategory) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'a34234',
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('insert', $NewCategory, ['hashref']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('NewCategory', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'a00121',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Class', ['CategoryPackage'], sub {
        my ($CategoryPackage) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {
            'Alias' => 'CategoryClass',
        });
        $Mockify->addMockWithReturnValue('packageObject', $CategoryPackage);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createMocks_getCategoryData {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
    $Mockery->extend($self->_createCommonMocks());

    $Mockery->register('SUT', ['Locale', 'System'], sub {
        my ($Locale, $System) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Catalog::API::CategoryService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocale', $Locale, ['string']);
        $Mockify->addMockWithReturnValue('_LoadRootObject', $System);
        $Mockify->addMockWithReturnValueAndParameterCheck('_getSpecialOffer', 0, ['object']);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Category', ['Shop', 'ParentCategory', 'SubCategory'], sub {
        my ($Shop, $Parent, $SubCategory) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias'                                  => 'alias',
            'Site'                                   => $Shop,
            'GUID'                                   => '123-456-7890',
            'ObjectID'                               => '08154711',
            'Name'                                   => 'name',
            'NameOrAlias'                            => 'name',
            'DisplayTitle'                           => 'title',
            'DisplayDescription'                     => 'description',
            'NavigationCaption'                      => 'nav caption',
            'WebPath'                                => 'http://web.path/',
            'WebUrl'                                 => 'http://web.url/',
            'ImageMedium'                            => 'mediumSizedTestImage',
            'ImageSmall'                             => 'smallSizedTestImage',
            'Parent'                                 => $Parent,
            'VisibleSubCategories'                   => [ $SubCategory ],
            'SubCategories'                          => [ $SubCategory ],
            'IsVisibleContentView'                   => 1,
            'IsVisible'                              => 0,
            'ReadPermissionFor'                      => '123',
            'UnityFacetedSearchShowFacetsOnCategory' => 'true',
            'FacetedSearchStartDepth'                => '3',

        });
        $Mockify->addMockWithReturnValueAndParameterCheck('uniqueChildAlias', 'testAlias', [{'string' => 'test identifier'}]);
        $Mockify->addMockWithReturnValue('renumberChildren', undef);
        return $Mockify->getMockObject();
    });
    $Mockery->register('ParentCategory', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'GUID' => '314-159-26',
            'Alias' => 'p1',
            'NavigationString' => 'use this string for navigation',
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('instanceOf', 1, [{'string' => 'Category'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('SubCategory', ['Class'], sub {
        my ($Class) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 's1',
            'GUID' => '132-557',
            'NavigationString' => 'another string for navigation',
        });
        $Mockify->addMockWithReturnValue('class', $Class);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Context', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValue('imageBaseUrl', 'http://image.base.url');
        $Mockify->addMockWithReturnValue('apiBaseUrl', 'http://api.base.url');
        $Mockify->addMockWithReturnValue('facetedSearchStartDepth', '3');
        $Mockify->addMockWithReturnValue('unityFacetedSearchShowFacetsOnCategory', 'true');
        return $Mockify->getMockObject();
    });
    $Mockery->register('Class', [], sub {
        my ($CategoryPackage) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('isSubClassOf', 1, [{'string' => 'Category'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('System', ['Groups'], sub {
        my ($Groups) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Site', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('child', $Groups, [{'string' => 'Groups'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Groups', ['Everyone'], sub {
        my ($Everyone) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', {});
        $Mockify->addMockWithReturnValueAndParameterCheck('child', $Everyone, [{'string' => 'Everyone'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('Everyone', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', {
            'ID' => 123,
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}



sub _createCommonMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('Locale', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Locale', {
            'LanguageID' => '1',
        });
        return $Mockify->getMockObject();
    });
    $Mockery->register('Shop', ['RootCategory'], sub {
        my ($RootCategory) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('child', $RootCategory, [{'string' => 'Categories'}]);
        return $Mockify->getMockObject();
    });
    $Mockery->register('RootCategory', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'Categories',
            'GUID' => '123-456-7890-root-category',
            'Name' => 'name',
            'NameOrAlias' => 'name',
            'DisplayTitle' => 'title',
            'DisplayDescription' => 'description',
            'NavigationCaption' => 'nav caption',
            'IsVisibleContentView' => 1,
            'IsVisible' => 1,
        });
        return $Mockify->getMockObject();
    });
    return $Mockery->assemble();
}



sub _createExpectedCategoryData {
    return {
        'categoryId' => '123-456-7890',
        'objectId'   => '08154711',
        'facetedSearchShowFacetsOnCategory' => 'true',
        'facetedSearchCategoryStartDepth' => '3',
        'visible' => 'false',
        'parent' =>
            {
                'rel' => 'p1',
                'href' => 'http://api.base.url/api/categories/314-159-26',
                'title' => 'use this string for navigation'
            },
        'images' =>
            [
                {
                    'url' => 'http://image.base.urlhttp://web.path//mediumSizedTestImage',
                    'classifier' => 'Medium'
                },
                {
                    'url' => 'http://image.base.urlhttp://web.path//smallSizedTestImage',
                    'classifier' => 'Small'
                }
            ],
        'pageTitle' => 'title',
        'specialOffer' => '0',
        'name' => 'name',
        'description' => 'description',
        'alias' => 'alias',
        'navigationCaption' => 'nav caption',
        'sfUrl' => 'http://web.url/',
        'subCategories' =>
            [
                {
                    'rel' => 's1',
                    'href' => 'http://api.base.url/api/categories/132-557',
                    'title' => 'another string for navigation'
                }
            ]
    };
}

__PACKAGE__->runScript();