#========================================================================================
# package      DE_EPAGES::RemoteSearch::UI::Shop
# state        private
#----------------------------------------------------------------------------------------
# description  Handles search request by issuing a request to the remote search service
#========================================================================================
package DE_EPAGES::RemoteSearch::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );
use DE_EPAGES::Content::API::Constants qw (MAX_NESTING_OF_CATEGORIES);
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # save layout settings
    my $PageType = LoadPageTypeByAlias( 'SF-FacetedSearch' );

    my $TemplateName = 'INC-SearchResults';
    my $TemplateTypeName = $Servlet->form->value( $TemplateName );
    $PageType->setTemplateType( $Shop, $TemplateName, $TemplateTypeName );

    # save message text
    my $hValues = $Servlet->form->form( $Shop, 'SearchSettings' );

    # EPG-8337: some settings are only availabe with certain features
    delete $hValues->{$_} foreach grep { !defined $hValues->{$_} } keys %$hValues;

    foreach my $hMessage ( @{$hValues->{Languages}} ) {
        $Shop->set({ RemoteNoResultMessage => $hMessage->{RemoteNoResultMessage} }, $hMessage->{LanguageID} );
    }

    # save other attributes
    $self->SUPER::Save($Servlet);

    $Shop->updatePageCache;
    return;
}

sub UnitySaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;

    my $ShopSettings;
    foreach my $ShopAttribute (qw(UnityFacetedSearchFilterCategory UnityFacetedSearchFilterManufacturer UnityFacetedSearchFilterPriceRange)) {
        if ($Form->value($ShopAttribute)) {
            $ShopSettings->{$ShopAttribute} = 1;
        } else {
            $ShopSettings->{$ShopAttribute} = 0;
        }
    }

    $Shop->set($ShopSettings);

    my @IsSearchFilterIDs = $Servlet->form->value('IsSearchFilter');
    my %IsSearchFilterIDs = map { $_ => 1 } @IsSearchFilterIDs;

    foreach my $ProductType (@{ $Shop->get('AllProductTypes') }) {
        foreach my $Attribute (@{ $ProductType->get('OwnUserDefinedAttributes') }) {
            my $IsSearchFilter = $IsSearchFilterIDs{$Attribute->id} ? 1 : 0;
            $Attribute->set({ 'IsSearchFilter' => $IsSearchFilter });
        }
    }

    # save other attributes
    $self->SUPER::Save($Servlet);

    return;
}

sub UnityViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my $ShowProductAttributeFilters = 0;

    foreach my $ProductType (@{ $Shop->get('AllProductTypes') }) {
        my $ShowProductType = 0;
        foreach my $Attribute (@{ $ProductType->get('OwnUserDefinedAttributes') }) {
            if ($Attribute->get('Type') eq 'PreDefLocalizedString') {
                $ShowProductAttributeFilters = 1;
                $ShowProductType = 1;
            }
        }
        $Servlet->vars('ShowProductType_' . $ProductType->alias, $ShowProductType);
    }

    $Servlet->vars('ShowProductAttributeFilters', $ShowProductAttributeFilters);
    $Servlet->vars('MaxNestingOfCategories', MAX_NESTING_OF_CATEGORIES);

    return;
}

1;
