package DE_EPAGES::Zettle::t::03_ZettleVariationOptionHelper;

use base DE_EPAGES::Shop::API::TestImportXML;
use Test::More;
use strict;
use warnings;

use DE_EPAGES::Product::API::Shop qw(GetProduct);

use DE_EPAGES::Zettle::API::WebService::VariationOptionHelper qw(BuildOptionDefinitions BuildVariantOptions);
use Data::Dumper;

sub shopAlias { return 'TestShop'; }
sub importXMLPath { return 'TestShop.xml'; }

sub test {
    my $self = shift;
    my $Shop = $self->shop;

    my $UseMaster = 1;
    my $ProductWithVariations = GetProduct($Shop, 'be_40401', $UseMaster);
    ok($ProductWithVariations, 'Product with variations loaded');
    ok(scalar(@{$ProductWithVariations->get('SubProducts')}) > 0, 'Test product has variations');

    my $OptionDefinitions = BuildOptionDefinitions($ProductWithVariations);
    # we should have Color x 1  and Size x 1 options with few a few values in both
    ok(scalar(@$OptionDefinitions) == 2, ' two option definitions');

    my ($Color) = grep { 'Color' eq $_->{name} } @$OptionDefinitions;
    ok(defined($Color), ' found color');
    ok(scalar(@{$Color->{'properties'}} == 2), ' color has two properties');

    my ($Size) = grep { 'Size' eq $_->{name} } @$OptionDefinitions;
    ok(defined($Size), ' found size');
    ok(scalar(@{$Size->{'properties'}} == 5), ' size has five properties');

    # build variation data for one sub product
    my $SubProduct = GetProduct($Shop, 'be_4040104003');
    my $VariantOptions = BuildVariantOptions($SubProduct);

    # we should have color and size defined with values from TestShop.xml
    my ($ColorVariant) = grep { 'Color' eq $_->{name} } @$VariantOptions;
    ok($ColorVariant->{'value'} eq $SubProduct->get('Color')->get('ValueOrAlias', $Shop->get('LanguageID')), ' color value matches');
    my ($SizeVariant) = grep { 'Size' eq $_->{name} } @$VariantOptions;
    ok($SizeVariant->{'value'} eq $SubProduct->get('Size')->get('ValueOrAlias', $Shop->get('LanguageID')), ' size value matches');

    done_testing();
}

__PACKAGE__->runScript;
1;
